open Datatypes
open Specif
open Computable_total_ordering

module Make = 
 functor (Domain:Computable_total_ordering.ORDERED_SET) ->
 struct 
  module Func_maps = Maps.Functional_maps(Domain)
  
  module Domain = Domain
  
  type domain = Domain.t
  
  type 'codomain alist_set =
    | Empty_alist
    | Cons_alist of domain * 'codomain * 'codomain alist_set
  
  (** val alist_set_rect : 'a2 -> (domain -> 'a1 -> 'a1 alist_set -> 'a2 ->
                           'a2) -> 'a1 alist_set -> 'a2 **)
  
  let rec alist_set_rect f f0 = function
    | Empty_alist -> f
    | Cons_alist (d, c, a0) -> f0 d c a0 (alist_set_rect f f0 a0)
  
  (** val alist_set_rec : 'a2 -> (domain -> 'a1 -> 'a1 alist_set -> 'a2 ->
                          'a2) -> 'a1 alist_set -> 'a2 **)
  
  let rec alist_set_rec f f0 = function
    | Empty_alist -> f
    | Cons_alist (d, c, a0) -> f0 d c a0 (alist_set_rec f f0 a0)
  
  type 'codomain t = 'codomain alist_set coq_sig
  
  (** val empty : unit -> 'a1 t **)
  
  let empty dummy =
    Empty_alist
  
  (** val alist_find : 'a1 alist_set -> domain -> 'a1 option **)
  
  let rec alist_find l key =
    match l with
      | Empty_alist -> None
      | Cons_alist (h, d, t0) ->
          (match Domain.compare key h with
             | LESS -> None
             | EQUAL -> Some d
             | GREATER -> alist_find t0 key)
  
  (** val find : domain -> 'a1 t -> 'a1 option **)
  
  let find key l =
    alist_find l key
  
  (** val alist_singleton : domain -> 'a1 -> 'a1 alist_set **)
  
  let alist_singleton key img =
    Cons_alist (key, img, Empty_alist)
  
  (** val singleton : domain -> 'a1 -> 'a1 t **)
  
  let singleton key img =
    alist_singleton key img
  
  (** val alist_add : 'a1 alist_set -> domain -> 'a1 -> 'a1 alist_set **)
  
  let rec alist_add l key img =
    match l with
      | Empty_alist -> Cons_alist (key, img, Empty_alist)
      | Cons_alist (h, d, t0) ->
          (match Domain.compare key h with
             | LESS -> Cons_alist (key, img, l)
             | EQUAL -> Cons_alist (key, img, t0)
             | GREATER -> Cons_alist (h, d, (alist_add t0 key img)))
  
  (** val add : domain -> 'a1 -> 'a1 t -> 'a1 t **)
  
  let add key img l =
    alist_add l key img
  
  (** val func_add : Func_maps.domain -> 'a1 -> (Func_maps.domain -> 'a1
                     option) -> Func_maps.domain -> 'a1 option **)
  
  let func_add key img f lookup =
    Func_maps.func_add key img f lookup
  
  (** val alist_remove : domain -> 'a1 alist_set -> 'a1 alist_set **)
  
  let rec alist_remove key m = match m with
    | Empty_alist -> Empty_alist
    | Cons_alist (head, image, tail) ->
        (match Domain.compare key head with
           | LESS -> m
           | EQUAL -> tail
           | GREATER -> Cons_alist (head, image, (alist_remove key tail)))
  
  (** val func_remove : domain -> (domain -> 'a1 option) -> domain -> 'a1
                        option **)
  
  let func_remove key f lookup =
    match Domain.compare key lookup with
      | LESS -> f lookup
      | EQUAL -> None
      | GREATER -> f lookup
  
  (** val remove : domain -> 'a1 t -> 'a1 t **)
  
  let remove key m =
    alist_remove key m
  
  (** val alist_foldi_right : (domain -> 'a1 -> 'a2 -> 'a2) -> 'a1 alist_set
                              -> 'a2 -> 'a2 **)
  
  let rec alist_foldi_right f l init =
    match l with
      | Empty_alist -> init
      | Cons_alist (c, d, t0) -> f c d (alist_foldi_right f t0 init)
  
  (** val foldi_right : (domain -> 'a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> 'a2 **)
  
  let foldi_right f l init =
    alist_foldi_right f l init
  
  (** val alist_foldi_left : (domain -> 'a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1
                             alist_set -> 'a2 **)
  
  let rec alist_foldi_left f init = function
    | Empty_alist -> init
    | Cons_alist (c, d, t0) -> alist_foldi_left f (f c init d) t0
  
  (** val foldi_left : (domain -> 'a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 t -> 'a2 **)
  
  let foldi_left f init l =
    alist_foldi_left f init l
  
  (** val alist_fold_right : ('a1 -> 'a2 -> 'a2) -> 'a1 alist_set -> 'a2 ->
                             'a2 **)
  
  let rec alist_fold_right f l init =
    match l with
      | Empty_alist -> init
      | Cons_alist (c, d, t0) -> f d (alist_fold_right f t0 init)
  
  (** val fold_right : ('a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> 'a2 **)
  
  let fold_right f l init =
    alist_fold_right f l init
  
  (** val alist_fold_left : ('a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 alist_set ->
                            'a2 **)
  
  let rec alist_fold_left f init = function
    | Empty_alist -> init
    | Cons_alist (c, d, t0) -> alist_fold_left f (f init d) t0
  
  (** val fold_left : ('a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 t -> 'a2 **)
  
  let fold_left f init l =
    alist_fold_left f init l
  
  (** val alist_le_decide : ('a1 -> 'a1 -> bool) -> 'a1 alist_set -> 'a1
                            alist_set -> bool **)
  
  let rec alist_le_decide codomain_le_decide l1 l2 =
    match l1 with
      | Empty_alist ->
          (match l2 with
             | Empty_alist -> true
             | Cons_alist (d, c, l3) -> false)
      | Cons_alist (d, c, a) ->
          (match l2 with
             | Empty_alist -> true
             | Cons_alist (d0, c0, a0) ->
                 (match Domain.compare d d0 with
                    | LESS ->
                        alist_le_decide codomain_le_decide a (Cons_alist (d0,
                          c0, a0))
                    | EQUAL ->
                        (match codomain_le_decide c c0 with
                           | true -> alist_le_decide codomain_le_decide a a0
                           | false -> false)
                    | GREATER -> false))
  
  (** val alist_map1 : ('a1 -> 'a2) -> 'a1 alist_set -> 'a2 alist_set **)
  
  let rec alist_map1 f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (h, d, t0) -> Cons_alist (h, (f d), (alist_map1 f t0))
  
  (** val map1 : ('a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let rec map1 f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (h, d, t0) -> Cons_alist (h, (f d), (map1 f t0))
  
  (** val func_map1 : ('a1 -> 'a2) -> (domain -> 'a1 option) -> domain -> 'a2
                      option **)
  
  let func_map1 f mapf lookup =
    match mapf lookup with
      | Some x -> Some (f x)
      | None -> None
  
  (** val alist_mapi1 : (domain -> 'a1 -> 'a2) -> 'a1 alist_set -> 'a2
                        alist_set **)
  
  let rec alist_mapi1 f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (h, d, t0) -> Cons_alist (h, (f h d), (alist_mapi1 f t0))
  
  (** val func_mapi1 : (domain -> 'a1 -> 'a2) -> (domain -> 'a1 option) ->
                       domain -> 'a2 option **)
  
  let func_mapi1 f mapf lookup =
    match mapf lookup with
      | Some x -> Some (f lookup x)
      | None -> None
  
  (** val mapi1 : (domain -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let rec mapi1 f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (h, d, t0) -> Cons_alist (h, (f h d), (mapi1 f t0))
  
  (** val alist_mapi1_opt : (domain -> 'a1 -> 'a2 option) -> 'a1 alist_set ->
                            'a2 alist_set **)
  
  let rec alist_mapi1_opt f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (d, c, tail) ->
        (match f d c with
           | Some c2 -> Cons_alist (d, c2, (alist_mapi1_opt f tail))
           | None -> alist_mapi1_opt f tail)
  
  (** val func_mapi1_opt : (domain -> 'a1 -> 'a2 option) -> (domain -> 'a1
                           option) -> domain -> 'a2 option **)
  
  let func_mapi1_opt f mapf lookup =
    match mapf lookup with
      | Some x -> f lookup x
      | None -> None
  
  (** val mapi1_opt : (domain -> 'a1 -> 'a2 option) -> 'a1 t -> 'a2 t **)
  
  let rec mapi1_opt f = function
    | Empty_alist -> Empty_alist
    | Cons_alist (d, c, tail) ->
        (match f d c with
           | Some c2 -> Cons_alist (d, c2, (mapi1_opt f tail))
           | None -> mapi1_opt f tail)
  
  (** val alist_map2 : ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) ->
                       'a1 alist_set -> 'a2 alist_set -> 'a3 alist_set **)
  
  let rec alist_map2 f1 f2 g l1 l2 =
    match l1 with
      | Empty_alist ->
          let rec alist_map3 = function
            | Empty_alist -> Empty_alist
            | Cons_alist (h, d, t0) -> Cons_alist (h, 
                (f2 d), (alist_map3 t0))
          in alist_map3 l2
      | Cons_alist (h1, d1, t1) ->
          let rec alist_map2aux l3 = match l3 with
            | Empty_alist -> Cons_alist (h1, (f1 d1),
                (let rec alist_map3 = function
                   | Empty_alist -> Empty_alist
                   | Cons_alist (h, d, t0) -> Cons_alist (h, 
                       (f1 d), (alist_map3 t0))
                 in alist_map3 t1))
            | Cons_alist (h2, d2, t2) ->
                (match Domain.compare h1 h2 with
                   | LESS -> Cons_alist (h1, (f1 d1),
                       (alist_map2 f1 f2 g t1 l3))
                   | EQUAL -> Cons_alist (h1, (g d1 d2),
                       (alist_map2 f1 f2 g t1 t2))
                   | GREATER -> Cons_alist (h2, (f2 d2), (alist_map2aux t2)))
          in alist_map2aux l2
  
  (** val func_map2 : ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) ->
                      (domain -> 'a1 option) -> (domain -> 'a2 option) ->
                      domain -> 'a3 option **)
  
  let func_map2 f1 f2 g mapf1 mapf2 lookup =
    match mapf1 lookup with
      | Some x1 ->
          (match mapf2 lookup with
             | Some x2 -> Some (g x1 x2)
             | None -> Some (f1 x1))
      | None ->
          (match mapf2 lookup with
             | Some x2 -> Some (f2 x2)
             | None -> None)
  
  (** val map2 : ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) -> 'a1 t
                 -> 'a2 t -> 'a3 t **)
  
  let map2 f1 f2 g l1 l2 =
    alist_map2 f1 f2 g l1 l2
  
  (** val alist_mapi2 : (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) ->
                        (domain -> 'a1 -> 'a2 -> 'a3) -> 'a1 alist_set -> 'a2
                        alist_set -> 'a3 alist_set **)
  
  let rec alist_mapi2 f1 f2 g l1 l2 =
    match l1 with
      | Empty_alist ->
          let rec alist_mapi3 = function
            | Empty_alist -> Empty_alist
            | Cons_alist (h, d, t0) -> Cons_alist (h, 
                (f2 h d), (alist_mapi3 t0))
          in alist_mapi3 l2
      | Cons_alist (h1, d1, t1) ->
          let rec alist_mapi2aux l3 = match l3 with
            | Empty_alist -> Cons_alist (h1, (f1 h1 d1),
                (let rec alist_mapi3 = function
                   | Empty_alist -> Empty_alist
                   | Cons_alist (h, d, t0) -> Cons_alist (h, 
                       (f1 h d), (alist_mapi3 t0))
                 in alist_mapi3 t1))
            | Cons_alist (h2, d2, t2) ->
                (match Domain.compare h1 h2 with
                   | LESS -> Cons_alist (h1, (f1 h1 d1),
                       (alist_mapi2 f1 f2 g t1 l3))
                   | EQUAL -> Cons_alist (h1, (g h1 d1 d2),
                       (alist_mapi2 f1 f2 g t1 t2))
                   | GREATER -> Cons_alist (h2, (f2 h2 d2),
                       (alist_mapi2aux t2)))
          in alist_mapi2aux l2
  
  (** val func_mapi2 : (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) ->
                       (domain -> 'a1 -> 'a2 -> 'a3) -> (domain -> 'a1
                       option) -> (domain -> 'a2 option) -> domain -> 'a3
                       option **)
  
  let func_mapi2 f1 f2 g mapf1 mapf2 lookup =
    match mapf1 lookup with
      | Some x1 ->
          (match mapf2 lookup with
             | Some x2 -> Some (g lookup x1 x2)
             | None -> Some (f1 lookup x1))
      | None ->
          (match mapf2 lookup with
             | Some x2 -> Some (f2 lookup x2)
             | None -> None)
  
  (** val mapi2 : (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) -> (domain
                  -> 'a1 -> 'a2 -> 'a3) -> 'a1 t -> 'a2 t -> 'a3 t **)
  
  let mapi2 f1 f2 g l1 l2 =
    alist_mapi2 f1 f2 g l1 l2
  
  (** val alist_mapi2_opt : (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2
                            -> 'a3 option) -> (domain -> 'a1 -> 'a2 -> 'a3
                            option) -> 'a1 alist_set -> 'a2 alist_set -> 'a3
                            alist_set **)
  
  let rec alist_mapi2_opt f1 f2 g l1 l2 =
    match l1 with
      | Empty_alist ->
          let rec alist_mapi1_opt0 = function
            | Empty_alist -> Empty_alist
            | Cons_alist (d, c, tail) ->
                (match f2 d c with
                   | Some c2 -> Cons_alist (d, c2, (alist_mapi1_opt0 tail))
                   | None -> alist_mapi1_opt0 tail)
          in alist_mapi1_opt0 l2
      | Cons_alist (h1, d1, t1) ->
          let rec alist_mapi2aux_opt l3 = match l3 with
            | Empty_alist ->
                (match f1 h1 d1 with
                   | Some d3 -> Cons_alist (h1, d3,
                       (let rec alist_mapi1_opt0 = function
                          | Empty_alist -> Empty_alist
                          | Cons_alist (d, c, tail) ->
                              (match f1 d c with
                                 | Some c2 -> Cons_alist (d, c2,
                                     (alist_mapi1_opt0 tail))
                                 | None -> alist_mapi1_opt0 tail)
                        in alist_mapi1_opt0 t1))
                   | None ->
                       let rec alist_mapi1_opt0 = function
                         | Empty_alist -> Empty_alist
                         | Cons_alist (d, c, tail) ->
                             (match f1 d c with
                                | Some c2 -> Cons_alist (d, c2,
                                    (alist_mapi1_opt0 tail))
                                | None -> alist_mapi1_opt0 tail)
                       in alist_mapi1_opt0 t1)
            | Cons_alist (h2, d2, t2) ->
                (match Domain.compare h1 h2 with
                   | LESS ->
                       (match f1 h1 d1 with
                          | Some d3 -> Cons_alist (h1, d3,
                              (alist_mapi2_opt f1 f2 g t1 l3))
                          | None -> alist_mapi2_opt f1 f2 g t1 l3)
                   | EQUAL ->
                       (match g h1 d1 d2 with
                          | Some d3 -> Cons_alist (h1, d3,
                              (alist_mapi2_opt f1 f2 g t1 t2))
                          | None -> alist_mapi2_opt f1 f2 g t1 t2)
                   | GREATER ->
                       (match f2 h2 d2 with
                          | Some d3 -> Cons_alist (h2, d3,
                              (alist_mapi2aux_opt t2))
                          | None -> alist_mapi2aux_opt t2))
          in alist_mapi2aux_opt l2
  
  (** val func_mapi2_opt : (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2 ->
                           'a3 option) -> (domain -> 'a1 -> 'a2 -> 'a3
                           option) -> (domain -> 'a1 option) -> (domain ->
                           'a2 option) -> domain -> 'a3 option **)
  
  let func_mapi2_opt f1 f2 g m1 m2 lookup =
    match m1 lookup with
      | Some c1 ->
          (match m2 lookup with
             | Some c2 -> g lookup c1 c2
             | None -> f1 lookup c1)
      | None ->
          (match m2 lookup with
             | Some c2 -> f2 lookup c2
             | None -> None)
  
  (** val mapi2_opt : (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2 -> 'a3
                      option) -> (domain -> 'a1 -> 'a2 -> 'a3 option) -> 'a1
                      t -> 'a2 t -> 'a3 t **)
  
  let mapi2_opt f1 f2 g l1 l2 =
    alist_mapi2_opt f1 f2 g l1 l2
 end

