open Datatypes
open Specif
open Computable_total_ordering

module Make : 
 functor (Domain:Computable_total_ordering.ORDERED_SET) ->
 sig 
  module Func_maps : 
   sig 
    type domain = Domain.t
    
    val func_add :
      domain -> 'a1 -> (domain -> 'a1 option) -> domain -> 'a1 option
    
    val func_remove :
      domain -> (domain -> 'a1 option) -> domain -> 'a1 option
   end
  
  module Domain : 
   sig 
    type t = Domain.t
    
    val compare : t -> t -> t comparison
   end
  
  type domain = Domain.t
  
  type 'codomain alist_set =
    | Empty_alist
    | Cons_alist of domain * 'codomain * 'codomain alist_set
  
  val alist_set_rect :
    'a2 -> (domain -> 'a1 -> 'a1 alist_set -> 'a2 -> 'a2) -> 'a1 alist_set ->
    'a2
  
  val alist_set_rec :
    'a2 -> (domain -> 'a1 -> 'a1 alist_set -> 'a2 -> 'a2) -> 'a1 alist_set ->
    'a2
  
  type 'codomain t = 'codomain alist_set coq_sig
  
  val empty : unit -> 'a1 t
  
  val alist_find : 'a1 alist_set -> domain -> 'a1 option
  
  val find : domain -> 'a1 t -> 'a1 option
  
  val alist_singleton : domain -> 'a1 -> 'a1 alist_set
  
  val singleton : domain -> 'a1 -> 'a1 t
  
  val alist_add : 'a1 alist_set -> domain -> 'a1 -> 'a1 alist_set
  
  val add : domain -> 'a1 -> 'a1 t -> 'a1 t
  
  val func_add :
    Func_maps.domain -> 'a1 -> (Func_maps.domain -> 'a1 option) ->
    Func_maps.domain -> 'a1 option
  
  val alist_remove : domain -> 'a1 alist_set -> 'a1 alist_set
  
  val func_remove : domain -> (domain -> 'a1 option) -> domain -> 'a1 option
  
  val remove : domain -> 'a1 t -> 'a1 t
  
  val alist_foldi_right :
    (domain -> 'a1 -> 'a2 -> 'a2) -> 'a1 alist_set -> 'a2 -> 'a2
  
  val foldi_right : (domain -> 'a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> 'a2
  
  val alist_foldi_left :
    (domain -> 'a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 alist_set -> 'a2
  
  val foldi_left : (domain -> 'a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 t -> 'a2
  
  val alist_fold_right : ('a1 -> 'a2 -> 'a2) -> 'a1 alist_set -> 'a2 -> 'a2
  
  val fold_right : ('a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> 'a2
  
  val alist_fold_left : ('a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 alist_set -> 'a2
  
  val fold_left : ('a2 -> 'a1 -> 'a2) -> 'a2 -> 'a1 t -> 'a2
  
  val alist_le_decide :
    ('a1 -> 'a1 -> bool) -> 'a1 alist_set -> 'a1 alist_set -> bool
  
  val alist_map1 : ('a1 -> 'a2) -> 'a1 alist_set -> 'a2 alist_set
  
  val map1 : ('a1 -> 'a2) -> 'a1 t -> 'a2 t
  
  val func_map1 :
    ('a1 -> 'a2) -> (domain -> 'a1 option) -> domain -> 'a2 option
  
  val alist_mapi1 : (domain -> 'a1 -> 'a2) -> 'a1 alist_set -> 'a2 alist_set
  
  val func_mapi1 :
    (domain -> 'a1 -> 'a2) -> (domain -> 'a1 option) -> domain -> 'a2 option
  
  val mapi1 : (domain -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t
  
  val alist_mapi1_opt :
    (domain -> 'a1 -> 'a2 option) -> 'a1 alist_set -> 'a2 alist_set
  
  val func_mapi1_opt :
    (domain -> 'a1 -> 'a2 option) -> (domain -> 'a1 option) -> domain -> 'a2
    option
  
  val mapi1_opt : (domain -> 'a1 -> 'a2 option) -> 'a1 t -> 'a2 t
  
  val alist_map2 :
    ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) -> 'a1 alist_set ->
    'a2 alist_set -> 'a3 alist_set
  
  val func_map2 :
    ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) -> (domain -> 'a1
    option) -> (domain -> 'a2 option) -> domain -> 'a3 option
  
  val map2 :
    ('a1 -> 'a3) -> ('a2 -> 'a3) -> ('a1 -> 'a2 -> 'a3) -> 'a1 t -> 'a2 t ->
    'a3 t
  
  val alist_mapi2 :
    (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) -> (domain -> 'a1 -> 'a2
    -> 'a3) -> 'a1 alist_set -> 'a2 alist_set -> 'a3 alist_set
  
  val func_mapi2 :
    (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) -> (domain -> 'a1 -> 'a2
    -> 'a3) -> (domain -> 'a1 option) -> (domain -> 'a2 option) -> domain ->
    'a3 option
  
  val mapi2 :
    (domain -> 'a1 -> 'a3) -> (domain -> 'a2 -> 'a3) -> (domain -> 'a1 -> 'a2
    -> 'a3) -> 'a1 t -> 'a2 t -> 'a3 t
  
  val alist_mapi2_opt :
    (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2 -> 'a3 option) -> (domain
    -> 'a1 -> 'a2 -> 'a3 option) -> 'a1 alist_set -> 'a2 alist_set -> 'a3
    alist_set
  
  val func_mapi2_opt :
    (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2 -> 'a3 option) -> (domain
    -> 'a1 -> 'a2 -> 'a3 option) -> (domain -> 'a1 option) -> (domain -> 'a2
    option) -> domain -> 'a3 option
  
  val mapi2_opt :
    (domain -> 'a1 -> 'a3 option) -> (domain -> 'a2 -> 'a3 option) -> (domain
    -> 'a1 -> 'a2 -> 'a3 option) -> 'a1 t -> 'a2 t -> 'a3 t
 end

